/***************************************************************************
                          k8ktest.h  -  description
                             -------------------
    begin                : Sun Jul 16 2000
    copyright            : (C) 2000 by Joris Struyve
    email                : joris@struyve.be
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* CVS information :
 *
 * $Id: k8ktest.h,v 1.26 2002/08/18 18:24:26 js Exp $
 *
 */

/* QT includes */
#ifndef K8KTEST
#define K8KTEST
#define NO_DEBUG
#include <qnamespace.h>
#include <qcolor.h>
#include <qcolordialog.h>
#include <qwindowsstyle.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qmenubar.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qpixmap.h>
#include <qmultilinedit.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qdialog.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qfile.h>
#include <qscrollbar.h>
#include <qpaintdevice.h>
#include <qpen.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qvaluelist.h>
#include <qcombobox.h>
#include <qfontdialog.h>
#include <qvalidator.h>

/* other includes */
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <stdarg.h>
#include <signal.h>
#include <sys/io.h>
#include <k8000.h>

/* defines */
#define VERSION "K8ktest 1.0.0 : pre DR6 (Development Release 6)."

/* prototypes */
void Signal(int i);
int Power(int num,int pw);
void ByteToHex(byte *ba,char **result,int len);
void ByteToBin(byte *ba,char **result,int len);
void HexToByte(const char *string,byte **result);

/* Class defintions */
class ApplicationWinow;
class AboutDialog;
class StatusBar;
class MenuBar;
class ToolBar;
class AnalogOutput;
class DigitalIo;
class AnalogInput;
class OptionsDialog;
class I2cDialog;
class Settings;
class ClickWidget;

/*************************************************************************************
* ApplicationWindow definition
*
*
*
**************************************************************************************/

class ApplicationWindow: public QMainWindow
{
Q_OBJECT
public:
	ApplicationWindow(QString con);
	~ApplicationWindow();

	void ShowCard(int cardnumber);
	void ShowChain(int chainnumber);
	Settings *settings;

public slots:
	void ShowAbout();
	void RestoreSettings();
	void SaveSettings();
	void Exit();
	void NullSlot();

	void ShowFontDialog();
	void ShowOptionDialog();
	void ShowI2cDialog();
	
	void Scan();
	void AllOutput();
	void AllIntput();
	void SetMinimum();
	void SetMaximum();

	
signals:

private:
	void SetNewOptions();

	OptionsDialog *optionsdialog;
	
	ToolBar *toolbar;
	StatusBar *statusbar;
	MenuBar *menu;
	
	QTimer *timer;
	
	AnalogOutput  *analogDAoutput[MaxDAchannel];
	AnalogOutput  *analogDACoutput[MaxDACchannel];
	AnalogInput  *analoginput[MaxADchannel];
	DigitalIo    *digitalio[MaxIOchannel];
	
	QFrame *mainframe;
	QFrame *analogoframe;
	QFrame *analogiframe;
	QFrame *digitalioframe;
	QFrame *buttonframe;
	
	QPushButton *quit;
	QPushButton *set_min;
	QPushButton *set_max;
	QPushButton *alli;
	QPushButton *allo;
	
};


/*************************************************************************************
* AboutDialog definition
*
*
*
**************************************************************************************/

class AboutDialog : public QDialog
{
Q_OBJECT
public:
	AboutDialog( QWidget *parent, const char *name );
	~AboutDialog();
	
	void resizeEvent(QResizeEvent * r_event);
private:
	QPushButton *ok;
};


/*************************************************************************************
* OptionsDialog definition
*
*
*
**************************************************************************************/

class OptionsDialog: public QDialog
{
Q_OBJECT
public:
	OptionsDialog(QWidget *parent,char *label);
	~OptionsDialog();
	
	void resizeEvent(QResizeEvent * r_event);
	int GetRefreshrate(){return refreshrate->text().toInt();};
	int GetI2cBusDelay(){return i2cbusdelay->text().toInt();};
	int GetLptport(){return (interface->currentItem()==0?-1:interface->currentItem());};
	
public slots:
	void hideEvent(QHideEvent *h);

signals:
	void Closed();

private:
	QWidget *par;
	
	QPushButton *ok;
	QPushButton *cancel;
	
	QLineEdit *i2cbusdelay;
	QLineEdit *refreshrate;
	QComboBox *interface;
	
	QLabel *l_i2cbusdelay;
	QLabel *l_refreshrate;
	QLabel *l_interface;

};

/*************************************************************************************
* I2cDialog definition
*
*
*
**************************************************************************************/

class I2cDialog: public QDialog
{
Q_OBJECT
public:
	I2cDialog(ApplicationWindow *parent,char *label);
	~I2cDialog();
	
	void resizeEvent(QResizeEvent * r_event);

public slots:
	void SendData();
	void ReadData();

signals:
	void Closed();

private:
	ApplicationWindow *par;
	
	QPushButton *read;
	QPushButton *send;
	QPushButton *close;
	
	QButtonGroup *dataformat;
	QRadioButton *hex_data;
	QRadioButton *char_data;

	QComboBox *chain;
	QComboBox *address;
	QLineEdit *data;
	QLineEdit *length;
	QLineEdit *received_hex_data;
	QLineEdit *received_bin_data;
	
	QGroupBox *send_data;
	QGroupBox *receive_data;

	QLabel *l_chain;
	QLabel *l_address;
	QLabel *l_data;
	QLabel *l_length;
	QLabel *l_received_hex_data;
	QLabel *l_received_bin_data;

};

/*************************************************************************************
* StatusBar definition
*
*
*
**************************************************************************************/

class StatusBar: public QStatusBar
{
Q_OBJECT
public:
	StatusBar(QWidget *parent,char *name);
	~StatusBar();

public slots:
	void SetStatusString(QString connectstring);
	void Blink();
	void BlinkTimer();

private:
	QWidget *par;

	QLabel *statusstring;
	QLabel *blinker;
	QTimer *timer;

};

/*************************************************************************************
* MenuBar definition
*
*
*
**************************************************************************************/

class MenuBar: public QMenuBar
{
Q_OBJECT
public:
	MenuBar(QWidget *parent,char *name);
	~MenuBar();

	bool GetSaveonexitState()
		{return this->options->isItemChecked(saveonexit);};

	void SetSaveonexitState(int state);

public slots:
	void ToggleSaveonexitState()
		{this->options->setItemChecked(saveonexit,!this->options->isItemChecked(saveonexit));};

private:
	QWidget *par;
	
	QPopupMenu *file;
	QPopupMenu *view;
	QPopupMenu *options;
	QPopupMenu *help;
	
	int saveonexit;
};

/*************************************************************************************
* ToolBar definition
*
*
*
**************************************************************************************/

class ToolBar: public QToolBar
{
Q_OBJECT
public:
	ToolBar(ApplicationWindow *parent,char *name);
	~ToolBar();

public slots:
	void ToggleCard();
	void ToggleChain();
	void SetCard(int cardnumber);
	void SetChain(int chainnumber);
	
private:
	QWidget *par;
	
	int current_card;
	
	QToolButton *quit;	
	QToolButton *refresh;
	QToolButton *options;
	QToolButton *save;
	
	QButtonGroup *card_buttons;
	QButtonGroup *chain_buttons;
	QPushButton  *card[4];
	QPushButton  *chain[5];
};

/*************************************************************************************
* DigitalIo definition
*
*
*
**************************************************************************************/
class DigitalIo: public QFrame
{
Q_OBJECT
public:
	DigitalIo(bool value,int io,QString label,QWidget *parent,char *name);
	~DigitalIo();

	void SetGeometry(int x,int y,int w,int h,int s);
	bool GetValue();
	bool GetIoType();

public slots:
	void SetValue(bool value);
	void ChangeValue();
	void Refresh();
	void SetIoType(bool type);

signals:
	void ValueChanged(bool value);

private:
	QWidget *par;
	
	QFrame *container;
	QToolButton *val;
	QLabel *cardlabel;
	QButtonGroup *io_config;
	QRadioButton *i_type;
	QRadioButton *o_type;

	int io_channel;
};

/*************************************************************************************
* AnalogOutput definition
*
*
*
**************************************************************************************/
class AnalogOutput: public QFrame
{
Q_OBJECT
public:
	AnalogOutput(int min,int max, int linestep,int pagestep,int value,int io,QString type,
				QString label,QWidget *parent,char *name);
	~AnalogOutput();

	void SetGeometry(int x,int y,int w,int h,int s);
	int GetValue();

public slots:
	void SetValue(int value);
	void ChangeValue(int value);
	void Refresh();

signals:
	void ValueChanged(int value);

private:
	QWidget *par;
	
	QFrame *container;
	QScrollBar *val;
	QLabel *valuelabel;
	QLabel *cardlabel;
	QString type;
	
	int io_channel;

};

/*************************************************************************************
* AnalogInput definition
*
*
*
**************************************************************************************/
class AnalogInput: public QFrame
{
Q_OBJECT
public:
	AnalogInput(int min,int max, int linestep,int pagestep,int value,int io,QString label,
				QWidget *parent,char *name);
	~AnalogInput();
	void SetGeometry(int x,int y,int w,int h,int s);
	int GetValue();

public slots:
	void ChangeValue(int value);
	void Refresh();

signals:
	void ValueChanged(int value);

private:
	QWidget *par;
	
	QFrame *container;
	QScrollBar *val;
	QLabel *valuelabel;
	QLabel *cardlabel;
	
	int io_channel;

};

/*************************************************************************************
* Settings definition
*
*
*
**************************************************************************************/
class Settings
{

public:
	Settings();
	~Settings();
	
	void SaveSettings();
	void RestoreSettings();

	int i2cbusdelay;
	int refreshrate;
	int lptport;
	int currentchain;
	int currentcard;
	int saveonexit;
	QString font;
	
	bool iotype[MaxIOchannel];
	bool iostate[MaxIOchannel];
	int aoDACstate[MaxDACchannel];
	int aoDAstate[MaxDAchannel];

private:
	QFile *configfile;
};

#endif
